/*
 * T-Plan Robot Enterprise, automated testing tool.
 * Copyright (C) 2009-2013 T-Plan Limited (http://www.t-plan.com),
 * Cloisters House, 8 Battersea Park Road, London, SW8 4BG, United Kingdom.
 * All rights reserved.
 */
package com.tplan.db;

import com.tplan.robot.scripting.DefaultJavaTestScript;
import com.tplan.robot.scripting.ScriptingContext;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.SQLException;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Superclass for all DB test scripts.
 */
abstract class AbstractDbScript extends DefaultJavaTestScript {
    // The connection error variable name

    public static final String VAR_DB_ERROR = "_DB_ERROR";
    // The key to store the current connection to the context with
    static final String CONTEXT_OBJECT_DB_URL = "CONTEXT_OBJECT_DB_URL";
    // The map of active connections. The key is the URL concatenated with other
    // parameters such as the driver and user name.
    static final Map<String, ConnectionBean> connectionPool = Collections.synchronizedMap(new HashMap<String, ConnectionBean>());

    /**
     * Get the current connection. This method must be called inside the
     * {@link DefaultJavaTestScript#test()} method.
     * @param reportFailure if there is no connection and the argument is <code>
     * true</code> the method creates an execution log and sets the exit code to 1.
     * @return the current connection.
     */
    protected Connection getCurrentConnection(boolean reportFailure) {
        ConnectionBean bean = getCurrentConnectionBean();
        return bean == null ? null : bean.connection;
    }

    /**
     * Get the current connection bean. This method must be called inside the
     * {@link DefaultJavaTestScript#test()} method.
     * @param reportFailure if there is no connection and the argument is <code>
     * true</code> the method creates an execution log and sets the exit code to 1.
     * @return the current connection.
     */
    protected ConnectionBean getCurrentConnectionBean(boolean reportFailure) {
        if (getContext() == null) {
            throw new IllegalStateException("ERROR: the context is null! This method may be called only inside "
                    + "the test() method of a subclass of this class!");
        }

        ConnectionBean bean = getCurrentConnectionBean();
        Connection conn = bean == null ? bean.connection : null;

        // Remove the _DB_ERROR variable if it exists
        getContext().getVariables().remove(VAR_DB_ERROR);

        // Test the connection
        if (conn == null) {
            String msg = "There's no database connection! Use the " + Connect.class.getName()
                    + " script to establish a connection first.";
            fail(msg, 1);
        }
        return bean;
    }

    private ConnectionBean getCurrentConnectionBean() {
        // Get the current connection
        ConnectionBean conn = null;
        String poolKey = (String) getContext().get(CONTEXT_OBJECT_DB_URL);
        if (poolKey != null) {
            conn = connectionPool.get(poolKey);
        }
        return conn;
    }

    /**
     * Set the current connection in the script context. The method also
     * configures the auto commit mode, creates a log message and sets the command
     * exit code 0 to indicate success.
     * @param conn a connection.
     * @param poolKey the connection pool key identifying the connection.
     * @param autoCommit the auto commit mode flag. If it is not null the connection
     * will be set to the desired mode using {@link Connection#setAutoCommit(boolean)}.
     * @param makeFail whether to make the script terminate on any DB error.
     * @param isNew true if the connection is a newly created one, false if it is a
     * previously created connection which was picked up from the connection pool.
     * @throws SQLException thrown eventually by the {@link Connection#setAutoCommit(boolean)} method.
     */
    protected void setCurrentConnection(ConnectionBean bean, String poolKey, Boolean autoCommit, Boolean makeFail, boolean isNew) throws SQLException {

        if (getContext() == null) {
            throw new IllegalStateException("ERROR: the context is null! This method may be called only inside "
                    + "the test() method of a subclass of this class!");
        }

        if (isNew) {
            connectionPool.put(poolKey, bean);
        }

        Connection conn = bean.connection;

        // Store the current connection pool key to the context. Any subsequent
        // DB calls will use it to identify the connection to be used.
        getContext().put(CONTEXT_OBJECT_DB_URL, poolKey);

        // Set the auto commit mode
        if (autoCommit != null) {
            conn.setAutoCommit(autoCommit);
        }

        bean.exitOnFailure = makeFail == null ? true : makeFail;

        // Create an execution log
        StringBuilder log = new StringBuilder();
        DatabaseMetaData metaData = conn.getMetaData();
        if (isNew) {
            log.append("Connected to ");
        } else {
            log.append("Picked up pooled connection to ");
        }
        log.append(metaData.getDatabaseProductName());
        log.append(" ").append(metaData.getDatabaseProductVersion());
        log.append(" (").append(metaData.getURL()).append(")");
        getContext().getExecutionLog().logInfo(log.toString());

        // Set the exit code to 0 (success)
        setVariable("_EXIT_CODE", "0");
    }

    /**
     * Fail routine - log the failure, set the _DB_ERROR variable and the exit
     * code and terminate the script if configured so.
     * @param error the error description.
     * @param exitCode the exit code.
     */
    protected void fail(String error, int exitCode) {
        setVariable(VAR_DB_ERROR, error);
        setVariable("_EXIT_CODE", Integer.toString(exitCode));

        ConnectionBean bean = getCurrentConnectionBean();
        if (bean != null && bean.exitOnFailure) {
            getContext().getExecutionLog().logSevere(error);
            try {
                exit(exitCode, error);
            } catch (IOException ex) {
                ex.printStackTrace();
            }
        } else {
            getContext().getExecutionLog().logWarning(error);
        }
    }
}
