/*
 * T-Plan Robot Enterprise, automated testing tool.
 * Copyright (C) 2009-2013 T-Plan Limited (http://www.t-plan.com),
 * Cloisters House, 8 Battersea Park Road, London, SW8 4BG, United Kingdom.
 * All rights reserved.
 */
package com.tplan.db;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.Map;

/**
 * This class registers as a shut down hook to the JVM to ensure that all
 * database connections are closed properly when the application exits.
 */
class DBShutdownHook extends Thread {

    // Static shutdown hook instance
    private static DBShutdownHook instance;

    // This class is a singleton.
    private DBShutdownHook() {
    }

    /**
     * Register the shut down hook in the current JVM. The hook is registered
     * just on the first call of this method and any later calls will have no effect.
     */
    static synchronized void register() {
        if (instance == null) {
            instance = new DBShutdownHook();
            Runtime.getRuntime().addShutdownHook(instance);
        }
    }

    /**
     * The shut down routine. It calls the close() method on every active
     * DB connection.
     */
    @Override
    public void run() {
        final Map<String, ConnectionBean> conns = AbstractDbScript.connectionPool;
        synchronized (conns) {
            for (ConnectionBean bean : conns.values()) {
                try {
                    bean.connection.close();
                } catch (SQLException ex) {
                    ex.printStackTrace();
                }
            }
            conns.clear();
        }
    }
}
